#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # File Manipulation with Azure Blob Storage
# 
# We try a few file manipulation between a local computer and a blob storage on Azure. It requires [azure-sdk-for-python](https://github.com/Azure/azure-sdk-for-python) and [pyenbc](http://www.xavierdupre.fr/app/pyenbc/helpsphinx/index.html). We first create a dummy file.

# In[1]:


import pandas, random
mat = [ {"x":random.random(), "y":random.random()} for i in range(0,1000)]
df = pandas.DataFrame(mat)
df.to_csv("randomxy.txt", sep="\t", encoding="utf8")


# We need credentials and to avoid having them in clear in the notebook, we use a HTML form:

# In[2]:


import pyquickhelper.ipythonhelper as ipy
params={"blob_storage":"hdblobstorage", "password":""}
ipy.open_html_form(params=params,title="credentials",key_save="blobservice")


# We stored the values in two variables in the workspace:

# In[3]:


blobstorage = blobservice["blob_storage"]
blobpassword = blobservice["password"]


# We need pyensae >= 1.2:

# In[4]:


import pyensae
import pyenbc
get_ipython().run_line_magic('load_ext', 'pyensae')
get_ipython().run_line_magic('load_ext', 'pyenbc')
pyensae.__version__, pyenbc.__version__


# In[5]:


get_ipython().run_line_magic('blob_open', '--help')


# We open a connection to the blob storage:

# In[6]:


cl, bs = get_ipython().run_line_magic('blob_open', '')
cl, bs


# We extract the available containers:

# In[7]:


l = get_ipython().run_line_magic('blob_containers', '')
l


# We get the content of one container:

# In[8]:


df = get_ipython().run_line_magic('blob_ls', 'hdblobstorage')
df.tail(n=5)


# In[9]:


get_ipython().run_line_magic('hd_wasb_prefix', '')


# In[10]:


cl.wasb_to_file("hdblobstorage", "velib_several_days")


# We upload the file we created in the first cell:

# In[11]:


get_ipython().run_line_magic('blob_up', 'randomxy.txt clusterensaeazure1/testpyenbc/randomxy.txt')


# We check the file is over there:

# In[12]:


get_ipython().run_line_magic('blob_ls', 'clusterensaeazure1/testpyenbc')


# We try an extended version:

# In[13]:


get_ipython().run_line_magic('blob_lsl', 'clusterensaeazure1/testpyenbc')


# If you need information not accessible through a magic command, you can use the variable ``bs`` (type [azure.storage.blobservice.BlobService](http://www.xavierdupre.fr/app/azure-sdk-for-python/helpsphinx/storage/blobservice.html#module-azure.storage.blobservice)):

# In[14]:


l=bs.get_block_list("clusterensaeazure1", "testpyenbc/randomxy.txt")
for _ in l.committed_blocks:
    print("size=",_.size, "id=",_.id)


# We download this again to the local computer:

# In[15]:


get_ipython().run_line_magic('blob_down', 'clusterensaeazure1/testpyenbc/randomxy.txt randomxx_copy.txt --overwrite')


# In[16]:


get_ipython().run_line_magic('lsr', 'r.*[.]txt')


# PIG scripts usually produce more than one output and it is convenient to merge them while downloading them. To test that, we upload a second time our file with a different names:

# In[17]:


get_ipython().run_line_magic('blob_up', 'randomxy.txt clusterensaeazure1/testpyenbc/randomxy2.txt')


# In[18]:


get_ipython().run_line_magic('blob_ls', 'clusterensaeazure1/testpyenbc')


# And we merge them:

# In[19]:


get_ipython().run_line_magic('blob_downmerge', 'clusterensaeazure1/testpyenbc randomall.txt --overwrite')


# We check the size of file ``randomall.txt`` is twice bigger:

# In[20]:


get_ipython().run_line_magic('lsr', 'r.*[.]txt')


# We finally remove the files from the blob storage:

# In[21]:


get_ipython().run_line_magic('blob_delete', 'clusterensaeazure1/testpyenbc/randomxy.txt')
get_ipython().run_line_magic('blob_delete', 'clusterensaeazure1/testpyenbc/randomxy2.txt')


# We check it disappeared:

# In[22]:


get_ipython().run_line_magic('blob_ls', 'clusterensaeazure1/testpyenbc/')


# And we close the connection:

# In[23]:


get_ipython().run_line_magic('blob_close', '')


# **END**

# In[24]: